@@ -1,7 +1,9 @@
 package DBIx::Diff::Schema;
 
-our $DATE = '2018-12-05'; # DATE
-our $VERSION = '0.093'; # VERSION
+our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
+our $DATE = '2020-04-20'; # DATE
+our $DIST = 'DBIx-Diff-Schema'; # DIST
+our $VERSION = '0.094'; # VERSION
 
 use 5.010001;
 use strict;
@@ -15,6 +17,7 @@ our @ISA = qw(Exporter);
 our @EXPORT_OK = qw(
                        list_columns
                        list_tables
+                       list_table_indexes
                        check_table_exists
                        diff_db_schema
                        diff_table_schema
@@ -117,8 +120,8 @@ sub list_tables {
     my $sth = $dbh->table_info(undef, undef, undef, undef);
     while (my $row = $sth->fetchrow_hashref) {
         my $name  = $row->{TABLE_NAME};
-        my $type  = $row->{TABLE_TYPE};
         my $schem = $row->{TABLE_SCHEM};
+        my $type  = $row->{TABLE_TYPE};
 
         if ($driver eq 'mysql') {
             # mysql driver returns database name as schema, so that's useless
@@ -146,6 +149,105 @@ sub list_tables {
     sort @res;
 }
 
+$SPEC{list_table_indexes} = {
+    v => 1.1,
+    summary => 'List indexes for a table in a database',
+    description => <<'_',
+
+General notes: information is retrieved from DBI's table_info().
+
+SQLite notes: autoindex for primary key is also listed as the first index, if it
+exists. This information is retrieved using "SELECT * FROM sqlite_master".
+Autoindex is not listed using table_info().
+
+_
+    args => {
+        %arg0_dbh,
+        %arg1_table,
+    },
+    args_as => "array",
+    result_naked => 1,
+};
+sub list_table_indexes {
+    my ($dbh, $wanted_table) = @_;
+
+    my $driver = $dbh->{Driver}{Name};
+
+    my @res;
+
+    if ($driver eq 'SQLite') {
+        my @wanted_tables;
+        if (defined $wanted_table) {
+            @wanted_tables = ($wanted_table);
+        } else {
+            @wanted_tables = list_tables($dbh);
+        }
+        for (@wanted_tables) { $_ = $1 if /.+\.(.+)/ }
+        my $sth = $dbh->prepare("SELECT * FROM sqlite_master");
+        $sth->execute;
+        while (my $row = $sth->fetchrow_hashref) {
+            next unless $row->{type} eq 'index';
+            next unless grep { $_ eq $row->{tbl_name} } @wanted_tables;
+            next unless $row->{name} =~ /\Asqlite_autoindex_.+_(\d+)\z/;
+            my $col_num = $1;
+            my @cols = list_columns($dbh, $row->{tbl_name});
+            push @res, {
+                name      => "(autoindex, primary key)",
+                table     => $row->{tbl_name},
+                columns   => [$cols[$col_num-1]{COLUMN_NAME}],
+                is_unique => 1,
+                is_pk     => 1,
+            };
+        }
+    }
+
+    my $sth = $dbh->table_info(undef, undef, undef, undef);
+    while (my $row = $sth->fetchrow_hashref) {
+        next unless $row->{TABLE_TYPE} eq 'INDEX';
+
+        my $table = $row->{TABLE_NAME};
+        my $schem = $row->{TABLE_SCHEM};
+
+        # match table name
+        if (defined $wanted_table) {
+            if ($driver eq 'mysql') {
+                # mysql driver returns database name as schema, so that's useless
+                $schem = '';
+                next unless $table eq $wanted_table;
+            } else {
+                if ($wanted_table =~ /(.+)\.(.+)/) {
+                    next unless $schem eq $1 && $table eq $2;
+                } else {
+                    next unless $table eq $wanted_table;
+                }
+            }
+        }
+
+        # parse index information
+        my $index_info = {};
+        if ($driver eq 'SQLite') {
+            next unless $row->{sqlite_sql};
+            #use DD; dd $row;
+            $index_info->{sqlite_sql} = $row->{sqlite_sql};
+            $row->{sqlite_sql} =~ s/\A\s*CREATE\s+(UNIQUE\s+)?INDEX\s+//is
+                or die "Can't extract CREATE INDEX statement in sqlite_sql: $row->{sqlite_sql}";
+            $index_info->{is_unique} = $1 ? 1:0;
+            $row->{sqlite_sql} =~ s/\A(\S+)\s+//s
+                or die "Can't extract index name from sqlite_sql: $row->{sqlite_sql}";
+            $index_info->{name} = $1;
+            $row->{sqlite_sql} =~ s/\AON\s*(\S+)\s*\(\s*(.+)\s*\)//s
+                or die "Can't extract indexed table+columns from sqlite_sql: $row->{sqlite_sql}";
+            $index_info->{table} = $table // $1;
+            $index_info->{columns} = [split /\s*,\s*/, $2];
+        } else {
+            die "Driver $driver is not yet supported";
+        }
+
+        push @res, $index_info;
+    }
+    sort @res;
+}
+
 $SPEC{list_columns} = {
     v => 1.1,
     summary => 'List columns of a table',
@@ -273,10 +375,10 @@ _
     "x.perinci.sub.wrapper.disable_validate_args" => 1,
 };
 sub diff_table_schema {
-    my $dbh1    = shift; no warnings ('void');require Scalar::Util;my $arg_err; ((defined($dbh1)) ? 1 : (($arg_err //= "Required but not specified"),0)) && ((Scalar::Util::blessed($dbh1)) ? 1 : (($arg_err //= "Not of type object"),0)); if ($arg_err) { die "diff_table_schema(): " . "Invalid argument value for dbh1: $arg_err" } # VALIDATE_ARG
-    my $dbh2    = shift; no warnings ('void');((defined($dbh2)) ? 1 : (($arg_err //= "Required but not specified"),0)) && ((Scalar::Util::blessed($dbh2)) ? 1 : (($arg_err //= "Not of type object"),0)); if ($arg_err) { die "diff_table_schema(): " . "Invalid argument value for dbh2: $arg_err" } # VALIDATE_ARG
-    my $table1  = shift; no warnings ('void');((defined($table1)) ? 1 : (($arg_err //= "Required but not specified"),0)) && ((!ref($table1)) ? 1 : (($arg_err //= "Not of type text"),0)); if ($arg_err) { die "diff_table_schema(): " . "Invalid argument value for table1: $arg_err" } # VALIDATE_ARG
-    my $table2  = shift // $table1; no warnings ('void');((defined($table2)) ? 1 : (($arg_err //= "Required but not specified"),0)) && ((!ref($table2)) ? 1 : (($arg_err //= "Not of type text"),0)); if ($arg_err) { die "diff_table_schema(): " . "Invalid argument value for table2: $arg_err" } # VALIDATE_ARG
+    my $dbh1    = shift; my $arg_err; { no warnings ('void');require Scalar::Util;((defined($dbh1)) ? 1 : (($arg_err //= "Required but not specified"),0)) && ((Scalar::Util::blessed($dbh1)) ? 1 : (($arg_err //= "Not of type object"),0)); if ($arg_err) { die "diff_table_schema(): " . "Invalid argument value for dbh1: $arg_err" } } # VALIDATE_ARG
+    my $dbh2    = shift; { no warnings ('void');((defined($dbh2)) ? 1 : (($arg_err //= "Required but not specified"),0)) && ((Scalar::Util::blessed($dbh2)) ? 1 : (($arg_err //= "Not of type object"),0)); if ($arg_err) { die "diff_table_schema(): " . "Invalid argument value for dbh2: $arg_err" } } # VALIDATE_ARG
+    my $table1  = shift; { no warnings ('void');((defined($table1)) ? 1 : (($arg_err //= "Required but not specified"),0)) && ((!ref($table1)) ? 1 : (($arg_err //= "Not of type text"),0)); if ($arg_err) { die "diff_table_schema(): " . "Invalid argument value for table1: $arg_err" } } # VALIDATE_ARG
+    my $table2  = shift // $table1; { no warnings ('void');((defined($table2)) ? 1 : (($arg_err //= "Required but not specified"),0)) && ((!ref($table2)) ? 1 : (($arg_err //= "Not of type text"),0)); if ($arg_err) { die "diff_table_schema(): " . "Invalid argument value for table2: $arg_err" } } # VALIDATE_ARG
 
     #$log->tracef("Comparing table %s vs %s ...", $table1, $table2);
 
@@ -350,8 +452,8 @@ _
     "x.perinci.sub.wrapper.disable_validate_args" => 1,
 };
 sub diff_db_schema {
-    my $dbh1 = shift; no warnings ('void');require Scalar::Util;my $arg_err; ((defined($dbh1)) ? 1 : (($arg_err //= "Required but not specified"),0)) && ((Scalar::Util::blessed($dbh1)) ? 1 : (($arg_err //= "Not of type object"),0)); if ($arg_err) { die "diff_db_schema(): " . "Invalid argument value for dbh1: $arg_err" } # VALIDATE_ARG
-    my $dbh2 = shift; no warnings ('void');((defined($dbh2)) ? 1 : (($arg_err //= "Required but not specified"),0)) && ((Scalar::Util::blessed($dbh2)) ? 1 : (($arg_err //= "Not of type object"),0)); if ($arg_err) { die "diff_db_schema(): " . "Invalid argument value for dbh2: $arg_err" } # VALIDATE_ARG
+    my $dbh1 = shift; my $arg_err; { no warnings ('void');require Scalar::Util;((defined($dbh1)) ? 1 : (($arg_err //= "Required but not specified"),0)) && ((Scalar::Util::blessed($dbh1)) ? 1 : (($arg_err //= "Not of type object"),0)); if ($arg_err) { die "diff_db_schema(): " . "Invalid argument value for dbh1: $arg_err" } } # VALIDATE_ARG
+    my $dbh2 = shift; { no warnings ('void');((defined($dbh2)) ? 1 : (($arg_err //= "Required but not specified"),0)) && ((Scalar::Util::blessed($dbh2)) ? 1 : (($arg_err //= "Not of type object"),0)); if ($arg_err) { die "diff_db_schema(): " . "Invalid argument value for dbh2: $arg_err" } } # VALIDATE_ARG
 
     my @tables1 = list_tables($dbh1);
     my @tables2 = list_tables($dbh2);
@@ -421,7 +523,7 @@ DBIx::Diff::Schema - Compare schema of two DBI databases
 
 =head1 VERSION
 
-This document describes version 0.093 of DBIx::Diff::Schema (from Perl distribution DBIx-Diff-Schema), released on 2018-12-05.
+This document describes version 0.094 of DBIx::Diff::Schema (from Perl distribution DBIx-Diff-Schema), released on 2020-04-20.
 
 =head1 SYNOPSIS
 
@@ -467,11 +569,13 @@ DBI database handle.
 
 Table name.
 
+
 =back
 
 Return value:  (any)
 
 
+
 =head2 db_schema_eq
 
 Usage:
@@ -499,11 +603,13 @@ DBI database handle for the first database.
 
 DBI database handle for the second database.
 
+
 =back
 
 Return value:  (any)
 
 
+
 =head2 diff_db_schema
 
 Usage:
@@ -552,11 +658,13 @@ DBI database handle for the first database.
 
 DBI database handle for the second database.
 
+
 =back
 
 Return value:  (any)
 
 
+
 =head2 diff_table_schema
 
 Usage:
@@ -602,11 +710,13 @@ Table name.
 
 Second table name (assumed to be the same as first table name if unspecified).
 
+
 =back
 
 Return value:  (any)
 
 
+
 =head2 list_columns
 
 Usage:
@@ -629,11 +739,48 @@ DBI database handle.
 
 Table name.
 
+
 =back
 
 Return value:  (any)
 
 
+
+=head2 list_table_indexes
+
+Usage:
+
+ list_table_indexes($dbh, $table) -> any
+
+List indexes for a table in a database.
+
+General notes: information is retrieved from DBI's table_info().
+
+SQLite notes: autoindex for primary key is also listed as the first index, if it
+exists. This information is retrieved using "SELECT * FROM sqlite_master".
+Autoindex is not listed using table_info().
+
+This function is not exported by default, but exportable.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<$dbh>* => I<obj>
+
+DBI database handle.
+
+=item * B<$table>* => I<str>
+
+Table name.
+
+
+=back
+
+Return value:  (any)
+
+
+
 =head2 list_tables
 
 Usage:
@@ -652,11 +799,13 @@ Arguments ('*' denotes required arguments):
 
 DBI database handle.
 
+
 =back
 
 Return value:  (any)
 
 
+
 =head2 table_schema_eq
 
 Usage:
@@ -692,6 +841,7 @@ Table name.
 
 Second table name (assumed to be the same as first table name if unspecified).
 
+
 =back
 
 Return value:  (any)
@@ -725,7 +875,7 @@ perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2018, 2017, 2015, 2014 by perlancar@cpan.org.
+This software is copyright (c) 2020, 2018, 2017, 2015, 2014 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
